/* This is free software. See COPYING for details. */
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/power.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <uWireM/uWireM.h>
#include <matrix8x8/matrix8x8.h>
#include <hmc5883l/hmc5883l.h>
#include "bitmap.h"
#include "calibrate.h"
#include "heading.h"

#define BRIGHTNESS 10 /* 0=dim, 15=max */

static void show(const int offset) {
   int i;
   uint8_t img[8];

   img[0] = img[1] = img[2] = 0;
   for(i = 0; i < BITMAP_HEIGHT; i++)
      img[i+3] = pgm_read_byte(_bitmap + offset + i);
   matrix8x8_draw(img);
}

int main(void) {
   calibration_t calib;
   hmc5883l_pos_t p;
   float hdg;

   power_timer1_disable(); /* turn off to save power */
   power_adc_disable();    /* likewise */
   uWireM_init();          /* set up I2C communication library */

   matrix8x8_init(BRIGHTNESS); /* set up 8x8 LED matrix */
   hmc5883l_init(); /* set up magnetometer */
   calibrate(&calib);

   show(BITMAP_NE);

   while(1) {
      hmc5883l_read(&p);
      hdg = heading(&p, &calib);
      if(hdg < 22.5) show(BITMAP_N);
      else if(hdg < 67.5) show(BITMAP_NE);
      else if(hdg < 112.5) show(BITMAP_E);
      else if(hdg < 157.5) show(BITMAP_SE);
      else if(hdg < 202.5) show(BITMAP_S);
      else if(hdg < 247.5) show(BITMAP_SW);
      else if(hdg < 292.5) show(BITMAP_W);
      else if(hdg < 337.5) show(BITMAP_NW);
      else show(BITMAP_N);
   }
   return 0;
}
